<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: txt.php 552 2008-12-18 12:12:16Z lks $
 */

/** zsource */
require_once 'zengine/classes/zsource.php';

/**
 * Чтение свойств страницы из формата txt
 *
 * @package    zengine
 * @subpackage zsource
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zsource_txt implements zsource
{
    /**
     * Удаляет страницу по пути $pagepath и все её дочерние страницы
     *
     * @param string $pagepath
     * @return boolean
     */
    public function delete ($pagepath)
    {
    	$filepath = self::$contentDir . '/' . $pagepath . '.' . zengine::config()->get('source.extension');
    	if (!@unlink($filepath)) {
    	    return false;
    	}
    	if (is_dir(self::$contentDir . '/' . $pagepath)) {
    	    @rmdir(self::$contentDir . '/' . $pagepath);
    	}
    	return true;
    }

    /**
     * Возвращает одномерный ассоциативный массив со всеми страницами, хранящимися в
     * папке content. Ключ и значение массива - путь к странице.
     *
     * @param string $parent путь к корневой странице, от которой будет осуществляться поиск
     * @return array
     */
    public function getAllPages ($parent = '') {
        $parent = '/' . trim(preg_replace("%[^a-zA-Z0-9-_/]+%", "", $parent), '/');

        if (!is_dir(self::$contentDir . $parent)) {
            return array();
        }

        $arPages = array();

        $extensionPattern = '%\.' . preg_quote(zengine::config()->get('source.extension'), '%') . '$%';
        $iterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator(self::$contentDir . $parent), RecursiveIteratorIterator::LEAVES_ONLY);
        foreach ($iterator as $file) {
            /* @var $file RecursiveDirectoryIterator */
            if (! $file->isFile() || substr($file->getFilename(), 0, 1) == '.' || strpos($iterator->getSubPathname(), '/.svn') !== false) {
                continue;
            }

            if (preg_match($extensionPattern, $file->getFilename())) {
                $pagepath = $this->filepath2pagepath($file->getPathname());
                $arPages[$pagepath] = $pagepath;
            }
        }

        return $arPages;
    }

    /**
     * Возвращает массив дочерних страниц
     *
     * @param string $pagepath путь к родительской странице
     * @return array
     */
    public function getChildren ($pagepath) {
        static $pregPattern = null;
        if (!$pregPattern) {
            $pregPattern = '%^((?!\.).*?)\.' . zengine::config()->get('source.extension') . '$%';
        }
        $arChildren = array();
        if (is_dir(self::$contentDir . $pagepath)) {
            $iterator = new DirectoryIterator(self::$contentDir . $pagepath);
            foreach ($iterator as $file) {
                /* @var $file DirectoryIterator */
                if (! preg_match($pregPattern, $file->getFilename())) {
                    continue;
                }
                $zpage = zpage::load($this->filepath2pagepath($file->getPathname()));
                if (! ($zpage->getProperty('publish') > ZENGINE_NOW)) {
                    $arChildren[] = $zpage;
                }
            }
        }
        return $arChildren;
    }


    /**
     * Формирует и возвращает меню сайта
     *
     * @return zmenu
     */
    public function getMenu () {
        if ($this->localCacheMenu) {
            return $this->localCacheMenu;
        }

        $menu = zengine::zcache()->get('menu', zcache::TTL_NEVER_EXPIRES);
        if (false !== $menu && $menu->getHash() == zengine::getInstance()->getSettingsHash()) {
            return $menu;
        }
        $menu = new zmenu();
        $menu->setHash(zengine::getInstance()->getSettingsHash());
        $this->appendMenu($menu, new DirectoryIterator(self::$contentDir));
        $menu->sort();
        zengine::zcache()->put('menu', $menu);
        return $this->localCacheMenu = $menu;
    }

    /**
     * Локальный кэш объекта меню сайта
     *
     * @var zmenu
     */
    private $localCacheMenu = null;

    /**
     * Возвращает true, если страница существует, в противном случае - false
     *
     * @return boolean
     */
    public function exists ($pagepath) {
    	return file_exists(self::$contentDir . '/' . ltrim($pagepath, '/') . '.' . zengine::config()->get('source.extension'));
    }

    /**
     * Возвращает свойства запрошенной страницы
     *
     * @param string $pagepath Путь к странице
     * @return array
     */
    public function load ($pagepath) {
        return $this->loadFromFile(self::$contentDir . $pagepath . '.' . zengine::config()->get('source.extension'));
    }

    /**
     * Возвращает время последней правки файла страницы
     *
     * @param string $pagepath путь к странице
     * @return int
     */
    public function mtime ($pagepath) {
        if (file_exists(self::$contentDir . $pagepath . '.' . zengine::config()->get('source.extension'))) {
            return filemtime(self::$contentDir . $pagepath . '.' . zengine::config()->get('source.extension'));
        }
        return 0;
    }

    /**
     * Возвращает свойства запрошенной страницы
     *
     * @param string $filename Путь к файлу с описанием страницы
     * @return array
     */
    public function loadFromFile ($filename) {
        $properties = $this->parse(file_get_contents($filename));
        if (empty($properties['publish'])) {
            $properties['publish'] = date('Y-m-d H:i:s', filemtime($filename));
        }
        return $properties;
    }

    /**
     * Возвращает свойства из строки
     *
     * @param string $content
     * @return array
     */
    public function parse ($content) {
        $this->properties = array();
        $this->properties['body'] = preg_replace_callback("%^@@([^=]+)=([^\r\n]+)\r?\n%m", array($this, 'extractProperty'), $content);
        return $this->properties;
    }

    /**
     * Сохраняняет в файл $pagepath.txt информацию о странице.
     *
     * @param string $pagepath путь к странице
     * @param array $properties ассоциативный массив свойств страницы
     * @return boolean
     *
     */
    public function save ($pagepath, array $properties) {
        $filepath = self::$contentDir . $pagepath . '.' . zengine::config()->get('source.extension');
        $content = array();
        if (isset($properties['body'])) {
            $body = $properties['body'];
            unset($properties['body']);
        }
        foreach ($properties as $property => $value) {
        	$content[] = '@@' . $property . '=' . $this->toSingleLine($value) . "\n";
        }
        $content[] = $body;

        if (!is_dir(dirname($filepath))) {
            mkdir(dirname($filepath), zengine::config()->get('directory.chmod'), true);
            if (self::$contentDir == dirname($filepath)) {
                file_put_contents(self::$contentDir . '/.htaccess', 'Deny from all');
            }
        } else {
            zutils::chmod(dirname($filepath), zengine::config()->get('directory.chmod'));
        }

        zutils::file_put_contents($filepath, implode($content));
        zutils::chmod($filepath, zengine::config()->get('file.chmod'));
        zutils::touch($filepath, time());
    	return true;
    }

    /**
     * Начинает транзакцию
     *
     */
    public function beginTransaction ()
    { }

    /**
     * Подтверждает изменения, внесенные во время транзакции
     *
     */
    public function commit ()
    { }

    /**
     * Откатывает изменения, внесенные во время транзакции
     *
     */
    public function rollBack ()
    { }

    /**
     * Сбрасывает кэш и обновляет необходимую информацию
     *
     */
    public function update () {
        zengine::zcache()->erase();
    }

    /**
     * Добавляет к пункту меню $zmenu подпункты
     *
     * @param zmenu $zmenu Пункт, к которому добавляются подпункты
     * @param DirectoryIterator $directory
     */
    private function appendMenu ($zmenu, $directory) {
        static $pregPattern = null;
        if (!$pregPattern) {
            $pregPattern = '%^((?!\.).*?)\.' . zengine::config()->get('source.extension') . '$%';
        }
    	foreach ($directory as $file) {
    	    /* @var $file DirectoryIterator */
    	    if ($file->isFile() && preg_match($pregPattern, $file->getFilename(), $matches)) {
    	        $page = zpage::load($this->filepath2pagepath($file->getPathname()));
    	        if ($page) {
    	            if (!$page->getPagepath()) {
    	                // Такого точно быть не должно
    	                return;
    	            }
    	            $submenu = $zmenu->appendPage($page);
    	            if (is_dir(self::$contentDir . $page->getPagepath())) {
    	                $this->appendMenu($submenu, new DirectoryIterator(self::$contentDir . $page->getPagepath()));
    	            }
    	        }
    	    }
    	}
    }

    /**
     * Преобразует путь к файлу в путь к странице
     *
     * @param string $filepath
     * @return string
     */
    private function filepath2pagepath ($filepath) {
        static $extLen = 0;
        if (!$extLen) {
            $extLen = -strlen(zengine::config()->get('source.extension')) - 1;
        }
    	return str_replace(DIRECTORY_SEPARATOR, '/', substr($filepath, strlen(self::$contentDir), $extLen));
    }


    /**
     * Метод используется в регулярном выражении функции load для получения свойств страницы
     *
     * @param array $matches
     * @return null
     */
    private function extractProperty ($matches) {
        $this->properties[strtolower($matches[1])] = $matches[2];
    	return null;
    }

    /**
     * Формирует из нескольких строк одну строку
     *
     * @param string $string
     * @return string
     */
    private function toSingleLine ($string) {
        return preg_replace("%\r?\n%", ' ', $string);
    }

    /**
     * Массив свойств для хранения полученных свойств страницы
     *
     * @var array
     */
    private $properties;

    /**
     * Папка, в которой хранятся все файлы страниц
     *
     * @var string
     */
    static private $contentDir = 'content';
}
