<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @subpackage ztemplate
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: functions.php 492 2008-11-12 13:42:53Z lks $
 */


/**
 * Представленные ниже функции являются короткими аналогами для использования
 * только в шаблоне сайта.
 *
 * Например, следующие строчки эквивалентны при использовании в шаблоне:
 *
 * <?php z_head(); ?>
 * <?php echo zengine::ztemplate()->head(); ?>
 * <?php echo $this->head(); ?>
 *
 * Если требуется как-то видоизменить выводимый текст, можно использовать более
 * длинный вариант с использованием $this.
 */

/**
 * Выводит содержимое тэга head для html-документа
 */
function z_head () {
	echo zengine::ztemplate()->head();
}


/**
 * Выводит название страницы
 */
function z_title () {
    echo zengine::ztemplate()->zpage->getTitle();
}

/**
 * Выводит заголовок страницы
 */
function z_h1 () {
    echo zengine::ztemplate()->zpage->getHeading();
}

/**
 * Выводит содержимое страницы
 */
function z_content () {
    echo zengine::ztemplate()->zpage->getBody();
}

/**
 * Выводит список дочерних страниц с их описанием
 *
 * @param string $sort Метод сортировки страниц
 * @param strin $open
 * @param strin $line
 * @param strin $close
 */
function z_subpages ($sort = 'mtime desc', $open = '', $line = '<a href="#link">#title</a><p>#description</p>', $close = '') {
    $html = zengine::ztemplate()->zpage->getCache('z_subpages');
    if (false === $html) {
        $html = zengine::ztemplate()->subpages($sort, $open, $line, $close);
        zengine::ztemplate()->zpage->cacheIt('z_subpages', $html);
    }
    echo $html;
}

/**
 * Выводит частично открытое меню
 */
function z_menu () {
    $html = zengine::ztemplate()->zpage->getCache('z_menu');
    if (false === $html) {
	   $html = zengine::ztemplate()->menu('minimal');
	   zengine::ztemplate()->zpage->cacheIt('z_menu', $html);
    }
    echo $html;
}

/**
 * Выводит полностью отктытое меню
 */
function z_fullmenu () {
    $html = zengine::ztemplate()->zpage->getCache('z_fullmenu');
    if (false === $html) {
	   $html = zengine::ztemplate()->menu('expanded');
	   zengine::ztemplate()->zpage->cacheIt('z_fullmenu', $html);
    }
    echo $html;
}

/**
 * Выводит меню в стиле cmsimple
 */
function cmsimple_menu () {
    $html = zengine::ztemplate()->zpage->getCache('cmsimple_menu');
    if (false === $html) {
        $html = zengine::ztemplate()->cmsimplemenu();
        zengine::ztemplate()->zpage->cacheIt('cmsimple_menu', $html);
    }
    echo $html;
}

/**
 * Выводит хлебные крошки
 */
function z_breadcrumbs () {
    $html = zengine::ztemplate()->zpage->getCache('breadcrumb');
    if (false === $html) {
        $html = zengine::ztemplate()->breadcrumb();
        zengine::ztemplate()->zpage->cacheIt('breadcrumb', $html);
    }
    echo $html;
}

/**
 * Выводит ссылку на карту сайта
 */
function z_sitemap () {
    echo zengine::getModule('zsitemap')->getLink();
}

/** Zebrum Lite 1.0.9 */

/**
 * Выводит ссылку на страницу облака тэгов
 */
function z_tags () {
    echo zengine::getModule('ztags')->getLink();
}

/**
 * Выводит список тэгов страницы
 *
 * @param string $template шаблон для вывода
 */
function z_pagetags ($template) {
    $html = zengine::ztemplate()->zpage->getCache('pagetags');
    if (false === $html) {
	   $html = zengine::ztemplate()->pagetags($template);
	   zengine::ztemplate()->zpage->cacheIt('pagetags', $html);
    }
    echo $html;
}

/**
 * Выводит хлебные крошки
 */
function z_tagcloud ($open = '<ul class="tagcloud">', $line = '<li><a href="#link">#title</a></li>', $close = '</ul>') {
    $html = zengine::zcache()->get('tagcloud');
    if (false === $html) {
        $html = zengine::ztemplate()->tagcloud($open, $line, $close);
        zengine::zcache()->put('tagcloud', $html);
    }
    echo $html;
}

/** Zebrum Lite 1.0.10 */

/**
 * Выводит мета-тэг со ссылкой на RSS-ленту
 *
 */
function z_rsshead () {
    echo '<link rel="alternate" type="application/rss+xml" title="' . htmlspecialchars(zengine::config()->get('rss.title', 'My site')) . '" href="~zfull/rss.xml" />';
}

/**
 * Выводит компактное меню
 */
function z_compactmenu () {
    $html = zengine::ztemplate()->zpage->getCache('z_compactmenu');
    if (false === $html) {
       $html = zengine::ztemplate()->compactmenu();
       zengine::ztemplate()->zpage->cacheIt('z_compactmenu', $html);
    }
    echo $html;
}

/** Zebrm Lite 1.0.11 */

/**
 * Выводит постраничную навигацию по внутренним страницам
 *
 * @param string $open
 * @param string $lineWithLink
 * @param string $lineWithoutLink
 * @param string $close
 * @param string $separator
 */
function z_pagenavigation ($open = '<div id="navigation"><span>Страницы:</span> ',
        $lineWithLink = '<a href="#link">#number</a>',
        $lineWithoutLink = '#number',
        $close = '</div>',
        $separator = ' | ') {
    $html = zengine::ztemplate()->zpage->getCache('z_pagenavigation');
    if (false === $html) {
        $html = zengine::ztemplate()->pagenavigation($open, $lineWithLink, $lineWithoutLink, $close, $separator);
        zengine::ztemplate()->zpage->cacheIt('z_pagenavigation', $html);
    }
    echo $html;
}

/** Zebrum Lite 1.1.0 */

/**
 * Выводит хлебные крошки
 *
 * @param string $before Текст перед хлебными крошками
 * @param string $separator Разделитель между страницами
 * @param string $after Текст после хлебных крошек
 * @param boolean $show_latest Показывать страницу, на которой находимся
 */
function z_breadcrumb ($before = '', $separator = ' &gt; ', $after = '', $show_latest = true) {
    $html = zengine::ztemplate()->zpage->getCache('breadcrumb');
    if (false === $html) {
        $html = zengine::ztemplate()->breadcrumb($before, $separator, $after, $show_latest);
        zengine::ztemplate()->zpage->cacheIt('breadcrumb', $html);
    }
    echo $html;
}

