<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zrss.php 491 2008-11-11 14:13:37Z lks $
 */

/**
 * Класс для создания rss-ленты
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @see        http://sitemaps.org/protocol.php
 */
class zmodule_zrss extends zmodule
{
    /**
     * Создает карту сайта, основываясь на меню, и возвращает ее
     *
     * @return string
     */
    public function getPage ($uri) {
        header('Content-Type: application/rss+xml; charset=' . zengine::config()->get('engine.charset'));
        /**
         * @todo реализовать формирование RSS с использованием XMLWriterx
         */
        $xml = zengine::zcache()->get('rss');
        if (false === $xml) {
            $xml = $this->plainCreate();
            zengine::zcache()->put('rss', $xml);
        }
        echo $xml;
        exit;
    }

    /**
     * Создает карту сайта без XMLWriter, основываясь на меню, и возвращает ее.
     *
     * @return string
     */
    private function plainCreate () {
        ob_start();

        /**
         * @todo возможность выбора языка
         */
        echo '<?xml version="1.0" encoding="' . zengine::config()->get('engine.charset') . '"?>
<rss version="2.0">
<channel>
<title><![CDATA[' . zengine::config()->get('rss.title', 'My site') . ']]></title>
<description><![CDATA[' . zengine::config()->get('rss.description', 'Description') . ']]></description>
<language>ru</language>
<link>http://' . $_SERVER['HTTP_HOST'] . '/</link>
<pubDate>' . date('r', strtotime(ZENGINE_NOW)) . '</pubDate>
<ttl>60</ttl>';

        /**
         * Перебираем все страницы и сохраняем, если дата публикации уже
         * наступила
         */
        $pages = array();
        $sections = preg_split("%\s*,\s*%", zengine::config()->get('rss.sections', ''), null, PREG_SPLIT_NO_EMPTY);
        if (empty($sections)) {
            $sections = array('');
        }

        foreach ($sections as $section) {
            foreach (zengine::zsource()->getAllPages($section) as $pagepath) {
                $properties = zengine::zsource()->load($pagepath);
                $properties['pagepath'] = $pagepath;
                if ($properties['publish'] > ZENGINE_NOW) {
                    continue;
                }
                $pages[] = zpage::createPage($properties);
            }
        }

        /**
         * Сортируем страницы
         */
        $sorter = new zpagesorter(zengine::config()->get('rss.order', 'sort asc, publish desc, title desc'));
        $sorter->sort($pages);

        /**
         * Выводим описание страниц
         */
        $limit = zengine::config()->get('rss.limit', 10);
        foreach ($pages as $page) {
            /* @var $page zpage */
            echo '<item>';
            $link = 'http://' . $_SERVER['HTTP_HOST'] . $page->getLink();
            echo '<title><![CDATA[' . $page->getTitle() . ']]></title>';
            echo '<description><![CDATA[' . $page->getDescription() . ']]></description>';
            echo '<link>' . $link . '</link>';
            echo '<pubDate>' . date('Y-m-d\TH:i:s+00:00', strtotime($page->getProperty('publish'))) . '</pubDate>';
            echo '</item>';
            if (!--$limit) {
                break;
            }
        }

        echo '</channel>
</rss>';
        $rss = ob_get_contents();
        ob_end_clean();
        return $rss;
    }
}
