<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: ztags.php 497 2008-11-13 07:54:17Z lks $
 */


/**
 * Карта сайта в стиле CMSimple
 *
 * @package    zengine
 * @subpackage zmodule
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zmodule_ztags extends zmodule {
    /**
     * Возвращает ссылку на запрошенный тэг
     *
     * @param string $tag
     * @return string
     */
    public function getTagLink ($tag) {
    	return '~/tags/' . urlencode($tag);
    }

    /**
     * Возвращает ссылку на карту сайта
     *
     * @return string
     */
    public function getLink () {
        if (zengine::getInstance()->getFilepath() == '/tags') {
            return $this->loadPage()->getMenu();
        }

        return '<a href="' . $this->loadPage()->getLink() .'">' . $this->loadPage(true)->getMenu() . '</a>';
    }

    /**
     * Возвращает страницу карты сайта в стиле CMSimple
     *
     * @param string $uri запрос к модулю
     * @return zpage
     */
    public function getPage ($uri) {
        // Если запрошена корневая страница как папка, то такой страницы не
        // существует
        if ($uri == $this->getParam('basePagepath') && !$this->getParam('isPage')) {
            return null;
        }

        $uri = substr($uri, strlen($this->getParam('basePagepath')) + 1);

        /**
         * Создаем страницу с необходимыми нам свойствами
         */
        $zpage = $this->loadPage();

        /**
         * Метод setBody принимает текст уже в конечной кодировке
         */
        $zpage->setBody($this->getBody(urldecode($uri)));
        return $zpage;
    }

    /**
     * Возвращает тело страницы карты сайты
     *
     * @return string
     */
    private function getBody ($tag) {
        ob_start();
        if (!empty($tag)) {
            $tags = array();
            $zmenu = zengine::zsource()->getMenu();
            $zmenu->extractTags($tags);
            $tagsPage = $this->loadPage();
            if (isset($tags[$tag])) {
                $tagsPage->setProperty('title', $tags[$tag]['name']);
                $tagsPage->setProperty('breadcrumb', $tags[$tag]['name']);


                echo $tagsPage->getProperty('__open', '');
                $line = $tagsPage->getProperty('__line', '<p><a href="#link">#title</a><br />#description</p>');
                foreach ($tags[$tag]['items'] as $item) {
                    $page = zpage::load($item->uri);
                    /* @var $page zpage */
                    $description = $page->getDescription();
                    if (empty($description)) {
                        $description = strip_tags(substr(ltrim($page->getBody(), " \n"), 0, strpos($page->getBody(), "\n")));
                    } else {
                        $description = $description;
                    }
                    $from = array('#link', '#title', '#description');
                    $to = array($page->getLink(), $page->getTitle(), $description);
                    echo str_replace($from, $to, $line);
                }
                echo $tagsPage->getProperty('__close', '</p>');
            }
        } else {
            echo $this->loadPage()->getBody();
            $this->tagcloud();
        }
        $body = ob_get_contents();
        ob_end_clean();
        return $body;
    }

    /**
     * Выводит облако тэгов
     *
     * @deprecated
     */
    public function tagcloud () {
        $tags = array();
        $zmenu = zengine::zsource()->getMenu();
        $zmenu->extractTags($tags);
        uasort($tags, array($this, 'tagcmp'));
        echo '<ul class="tagcloud">';
        foreach ($tags as $link => $tag) {
            echo '<li><a href="' . $this->getTagLink($link) . '">' . $tag['name'] . '</a></li>';
        }
        echo '</ul>';
    }

    /**
     * Сортирует тэги между собой
     *
     * @param array $left
     * @param array $right
     * @return int
     */
    public function tagcmp ($left, $right) {
        if ($left['count'] == $right['count']) {
            return strcmp($left['name'], $right['name']);
        }
        return ($left['count'] > $right['count'] ? -1 : 1);
    }

    /**
     * Возвращает объект страницы списка тэгов без содержимого
     *
     * @return zpage
     */
    private function loadPage ($reload = false) {
        if (isset($this->_zpage) && !$reload) {
            return $this->_zpage;
        }

        $properties = array();
        $properties['breadcrumb'] = $properties['title'] = $this->getParam('title', 'Tags');
        $properties['pagepath'] = $this->getParam('basePagepath');
        $properties['filter'] = 'asis';
        $this->_zpage = zpage::createPage($properties);
        return $this->_zpage;
    }

    /**
     * Объект загруженной страницы
     *
     * @var zpage
     */
    private $_zpage;
}